/*************************************
**function : Port test(output)-using ODR Register
           : Port test(input)-using IDR register
           ; LED Control Using External Interrupt 0, 1
**date     : 2022/03/1
**programmed by ksc
***************************************/
/***************************************
**define reset and clock control RCC
**RCC_base_address 0x40021000
***************************************/
#define RCC_CR_address        0x40021000  //clock control register
#define RCC_CFGR_address      0x40021004  //clock configuration register
#define RCC_CIR_address       0x40021008  //clock interrupt register
#define RCC_APB2RSTR_address  0x4002100C  //APB2 peripheral reset register
#define RCC_APB1RSTR_address  0x40021010  //APB2 peripheral reset register 
#define RCC_AHBENR_address    0x40021014  // AHB peripheral clock enable register
#define RCC_APB2ENR_address   0x40021018  // APB2 peripheral clock enable register
#define RCC_APB1ENR_address   0x4002101C  // APB2 peripheral clock enable register
#define RCC_BDCR_address      0x40021020  // backup domain control register
#define RCC_CSR_address       0x40021020  // control/status register
/********************************************
//Flash Memory Control
*********************************************/
#define Flash_ACR_address 0x40022000

/******************************************
**define GPIOA register address
**portA_base_address  0x40010c00
*******************************************/
#define GPIOA_CRL_address    0x40010800
#define GPIOA_CRH_address    0x40010804
#define GPIOA_IDR_address    0x40010808
#define GPIOA_ODR_address    0x4001080c
#define GPIOA_BSRR_address   0x40010810
#define GPIOA_BRR_address    0x40010814
#define GPIOA_CLKR_address   0x40010818

/*******************************************
**define GPIOB register address
**portB_base_address  0x40010c00
*******************************************/
#define GPIOB_CRL_address    0x40010c00
#define GPIOB_CRH_address    0x40010c04
#define GPIOB_IDR_address    0x40010c08
#define GPIOB_ODR_address    0x40010c0c
#define GPIOB_BSRR_address   0x40010c10
#define GPIOB_BRR_address    0x40010c14
#define GPIOB_CLKR_address   0x40010c18
/**********************************************
**NVIC Register
**NVIC Base Address :0xE000E100
**ISER(interrupt set enable register)
**ICER(interrupt clear enable register)
**ISPR(interrupt set pending register)
***********************************************/
#define ISER        0xE000E100
#define ICER        0xE000E180
#define ISPR        0xE000E200
#define IPR         0xE000E404
/******************************************************
**System Control Block Register
**Base address : 0xE000D00
VTOR(Vector Table Offset register)                     0xE000ED08
AIRCR(Application Interrupt/Reset Control register)    0xE000ED0C
*******************************************************/
#define VTOR      0xE000ED08
#define AIRCR     0xE000ED0C
/************************************************************
**AFIO Base address                                         :: 0x40010000
AFIO_EVCR(Event control register)                           :: 0ffset  :0x00
AFIO_MAPR(AF remap and debug I/O configuration register)    :: offset : 0x04
AFIO_EXTICR1(External interrupt configuration register 1)   :: offset : 0x08
AFIO_EXTICR2(External interrupt configuration register 2)   :: offset : 0x0C
AFIO_EXTICR3(External interrupt configuration register 3)   :: offset : 0x10
AFIO_EXTICR4(External interrupt configuration register 4)   :: offset : 0x14
AFIO_MAPR2(AF remap and debug I/O configuration register2)  :: offset : 0x1C
***************************************************************/
#define     AFIO_EVCR           0x40010000
#define     AFIO_MAPR           0x40010004
#define     AFIO_EXTICR1        0x40010008
#define     AFIO_EXTICR2        0x4001000C
#define     AFIO_EXTICR3        0x40010010
#define     AFIO_EXTICR4        0x40010014
#define     AFIO_MAPR2          0x4001001C
/**************************************************************
**External interrupt Base address                 ::0x4001 0400
**EXTI_IMR(Interrupt mask register)               ::offset :  0x00
**EXTI_EMR(Event mask register)                   ::offset :  0x04
**EXTI_RTSR(Rising trigger selection register)    ::offset :  0x08
**EXTI_FTSR(Falling trigger selection register)   ::offset  : 0x0C
**EXTI_SWIER(Software interrupt event register)   ::offset  : 0x10
**EXTI_PR(Pending register)                       ::offset  : 0x14
**********************************************************************/
#define   EXTI_IMR      0x40010400
#define   EXTI_EMR      0x40010404
#define   EXTI_RTSR     0x40010408
#define   EXTI_FTSR     0x4001040C
#define   EXTI_SWIER    0x40010410
#define   EXTI_PR       0x40010414
unsigned int key_flag;
void delay(void);
void LED(unsigned int);
void EXTI0_IRQHandler(void)
{
 key_flag=1;
  (*(volatile unsigned *)EXTI_PR=0x00000001);//clear pending
}

void EXTI1_IRQHandler(void)
{
 key_flag=2;
  (*(volatile unsigned *)EXTI_PR=0x00000002);//clear pending
}

main(){
/*************************************************************************/
// Initialize CLOCK Parameter
/*************************************************************************/  
(*(volatile unsigned *)RCC_CR_address)|=0x00010000;    //HSE ON enable
(*(volatile unsigned *)Flash_ACR_address)|=0x00000002; //set flash access latency
(*(volatile unsigned *)RCC_CFGR_address)|=0x00000400;  //PPRE2=0,PPRE1=/2,HPERE=0
(*(volatile unsigned *)RCC_CFGR_address)|=0x00110000; //PLL SOURCE =HSE, PLL XTPRE=0, PLLMUL= x6
(*(volatile unsigned *)RCC_CR_address)|=0x01010000;    //PLL enable
(*(volatile unsigned *)RCC_CIR_address)|=0x00000000;   // all clock interrupt disable
(*(volatile unsigned *)RCC_CFGR_address)|=0x00000002; //PLL selected as system clock

/**********************************************************************/ 
//enable AFIO CLOCK
/**********************************************************************/ 
(*(volatile unsigned *)RCC_APB2ENR_address)|=0x01;


 /**********************************************************************/ 
//enable port A for key in, enable port B for LED out
/**********************************************************************/ 
(*(volatile unsigned *)RCC_APB2ENR_address)|=0x0C;    //PortB &PortA enable

/**************************************************************************/
//key : portA    bit0, bit1
//CNF : Floating input input
//mode : input mode
/**************************************************************************/
(*(volatile unsigned *)GPIOA_CRL_address)|=0x00000044;
(*(volatile unsigned *)GPIOA_CRH_address)|=0x00000000;

/**************************************************************************/
//** PORTB initialization
//CNF : general purpose open drain output
//Mode : output mode(open drain, max speed : 50MHz)
(*(volatile unsigned *)GPIOB_CRL_address)|=0x00700000; //set bit5 mode&cnf

/**************************************************************************/
//** PORTB initialization
//CNF : general purpose open drain output
//Mode : output mode(open drain, max speed : 50MHz)
/**************************************************************************/
(*(volatile unsigned *)GPIOB_CRH_address)|=0x00000077; //set bit8,9 mode&cnf

/**************************************************************************/
//External Interrupt Configuration register1(AFIO_EXTICR1)
//0000: PA[x] pin
//0001: PB[x] pin
//0010: PC[x] pin
//0011: PD[x] pin
//0100: PE[x] pin
//0101: PF[x] pin
//0110: PG[x] pin
/***************************************************************************/
(*(volatile unsigned *)AFIO_EXTICR1)|=0x00000000;

/***************************************************************************/
//Interrupt Mask Register
//Rising trigger selection register
//Falling trigger selection register
//Pending register
/****************************************************************************/
(*(volatile unsigned *)EXTI_IMR)|=0x00000003;
(*(volatile unsigned *)EXTI_RTSR)|=0x00000003;
(*(volatile unsigned *)EXTI_FTSR)|=0x00000000;
(*(volatile unsigned *)EXTI_PR)|=0x00000003;  //clear all interrupt

/****************************************************************************/
//VTOR(vector table offset) Configuration 
/****************************************************************************/
// vector table base address=0x08000000
(*(volatile unsigned *)VTOR)|=0x00000000;   
//AIRCR|=0x05FA0000;
// no group priority, only sub priority
(*(volatile unsigned *)AIRCR)|=0x0700;      

/******************************************************************************/
//IPR(Interrupt Priority Register) Configuration
/******************************************************************************/
(*(volatile unsigned *)IPR)|=0x00000000;

/****************************************************************************/
//enable IRQ0,1
(*(volatile unsigned *)ISER)|=0x0000c0;  

/*******************************************************************************/
//Initialization outport
/*******************************************************************************/
(*(volatile unsigned *)GPIOB_ODR_address)=0x0320;



while(1){
 if(key_flag==1) {
   LED(5);
  key_flag=0;
  }
  else if(key_flag==2){
    LED(10);
    key_flag=0;
  }
 else (*(volatile unsigned *)GPIOB_ODR_address)=0x0320;
}
}
void delay(void){
unsigned int i,j;
for(i=0;i<2000;i++) for(j=0;j<2000;j++);
}

void LED(unsigned int num){
unsigned int i;
for(i=0;i<num;i++){
  (*(volatile unsigned *)GPIOB_ODR_address)=0x0320;
  delay();
  (*(volatile unsigned *)GPIOB_ODR_address)=0x0000;
  delay();
}

}
